@@ -396,9 +396,117 @@ float findsplinemin(splineptr thespline, XPoint *upoint)
    return tval;
 }
 
-/*----------------------------------------------------------------------------*/
-/* Find closest point of a polygon to the cursor			      */
-/*----------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Convert a polygon to a Bezier curve path				*/
+/* Curve must be selected and there must be only one selection.		*/
+/*									*/
+/* Note that this routine will draw inside the perimeter of a convex	*/
+/* hull.  A routine that places spline endpoints on the polygon		*/
+/* vertices will draw outside the perimeter of a convex hull.  An	*/
+/* optimal algorithm presumably zeros the total area between the curve	*/
+/* and the polygon (positive and negative), but I haven't worked out	*/
+/* what that solution is.  The algorithm below seems good enough for	*/
+/* most purposes.							*/
+/*----------------------------------------------------------------------*/
+
+void converttocurve()
+{
+   genericptr *ggen;
+   splineptr *newspline;
+   polyptr thispoly;
+   pathptr *newpath;
+   short *newselect;
+   XPoint firstpoint, lastpoint, initpoint;
+   int i, numpoints;
+
+   if (areawin->selects != 1) return;
+
+   thispoly = TOPOLY(topobject->plist + (*areawin->selectlist));
+   if (ELEMENTTYPE(thispoly) != POLYGON) return;
+   if (thispoly->number < 3) return;	/* Will not convert */
+
+   standard_element_delete(ERASE);
+   if ((thispoly->style & UNCLOSED) && (thispoly->number == 3)) {
+      NEW_SPLINE(newspline, topobject);
+      splinedefaults(*newspline, 0, 0);
+      (*newspline)->ctrl[0] = thispoly->points[0];
+      (*newspline)->ctrl[1] = thispoly->points[1];
+      (*newspline)->ctrl[2] = thispoly->points[1];
+      (*newspline)->ctrl[3] = thispoly->points[2];
+   }
+   else {
+      numpoints = thispoly->number;
+
+      /* If the polygon is closed but the first and last points	*/
+      /* overlap, treat the last point as if it doesn't exist.	*/
+
+      if (!(thispoly->style & UNCLOSED))
+         if ((thispoly->points[0].x == thispoly->points[thispoly->number - 1].x)
+		&& (thispoly->points[0].y ==
+		thispoly->points[thispoly->number - 1].y))
+	    numpoints--;
+
+      NEW_PATH(newpath, topobject);
+      pathdefaults(*newpath, 0, 0);
+      (*newpath)->style = thispoly->style;
+
+      if (!(thispoly->style & UNCLOSED)) {
+	 lastpoint = thispoly->points[numpoints - 1];
+	 initpoint.x = (lastpoint.x + thispoly->points[0].x) / 2;
+	 initpoint.y = (lastpoint.y + thispoly->points[0].y) / 2;
+	 firstpoint.x = (thispoly->points[0].x
+		+ thispoly->points[1].x) / 2;
+	 firstpoint.y = (thispoly->points[0].y
+			+ thispoly->points[1].y) / 2;
+
+         NEW_SPLINE(newspline, (*newpath));
+	 splinedefaults(*newspline, 0, 0);
+	 (*newspline)->ctrl[0] = initpoint;
+	 (*newspline)->ctrl[1] = thispoly->points[0];
+	 (*newspline)->ctrl[2] = thispoly->points[0];
+	 (*newspline)->ctrl[3] = firstpoint;
+         calcspline(*newspline);
+      }
+      else
+         firstpoint = thispoly->points[0];
+
+      for (i = 0; i < numpoints - ((!(thispoly->style & UNCLOSED)) ?
+		2 : 3); i++) {
+	 lastpoint.x = (thispoly->points[i + 1].x
+		+ thispoly->points[i + 2].x) / 2;
+	 lastpoint.y = (thispoly->points[i + 1].y
+			+ thispoly->points[i + 2].y) / 2;
+
+         NEW_SPLINE(newspline, (*newpath));
+	 splinedefaults(*newspline, 0, 0);
+	 (*newspline)->ctrl[0] = firstpoint;
+	 (*newspline)->ctrl[1] = thispoly->points[i + 1];
+	 (*newspline)->ctrl[2] = thispoly->points[i + 1];
+	 (*newspline)->ctrl[3] = lastpoint;
+	 firstpoint = lastpoint;
+         calcspline(*newspline);
+      }
+      if (!(thispoly->style & UNCLOSED))
+	 lastpoint = initpoint;
+      else
+	 lastpoint = thispoly->points[i + 2];
+
+      NEW_SPLINE(newspline, (*newpath));
+      splinedefaults(*newspline, 0, 0);
+      (*newspline)->ctrl[0] = firstpoint;
+      (*newspline)->ctrl[1] = thispoly->points[i + 1];
+      (*newspline)->ctrl[2] = thispoly->points[i + 1];
+      (*newspline)->ctrl[3] = lastpoint;
+   }
+   calcspline(*newspline);
+   calcbbox(areawin->topinstance);
+   setoptionmenu();
+   drawarea(NULL, NULL, NULL);
+}
+
+/*----------------------------------------------------------------------*/
+/* Find closest point of a polygon to the cursor			*/
+/*----------------------------------------------------------------------*/
 
 short closepointdistance(polyptr curpoly, XPoint *cursloc, short *mindist)
 {
@@ -1428,6 +1536,15 @@ void calcbboxinst(objinstptr thisinst)
 	 }
 	 calcbboxsingle(gelem, thisinst, &llx, &lly, &urx, &ury);
       }
+
+      /* If we have a clipmask, the clipmask is used to calculate the	*/
+      /* bounding box, not the element it is masking.			*/
+
+      switch(ELEMENTTYPE(*gelem)) {
+	 case POLYGON: case SPLINE: case ARC: case PATH:
+	    if (TOPOLY(gelem)->style & CLIPMASK) gelem++;
+	    break;
+      }
    }
 
    thisinst->bbox.lowerleft.x = llx;
@@ -1569,6 +1686,14 @@ void calcbboxvalues(objinstptr thisinst, genericptr *newelement)
 	    }
 	 }
 	 calcbboxsingle(bboxgen, thisinst, &llx, &lly, &urx, &ury);
+
+	 if (newelement == NULL)
+	    switch(ELEMENTTYPE(*bboxgen)) {
+	       case POLYGON: case SPLINE: case ARC: case PATH:
+	          if (TOPOLY(bboxgen)->style & CLIPMASK)
+		     bboxgen++;
+		  break;
+	    }
       }
 nextgen:
       if (newelement != NULL) break;
@@ -2138,9 +2263,9 @@ void UDrawBBox()
 		worig.x, worig.y);
 }
 
-/*-------------------------------------------------------------------------*/
-/* Fill and/or draw a border around the stroking path			   */
-/*-------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Fill and/or draw a border around the stroking path			*/
+/*----------------------------------------------------------------------*/
 
 void strokepath(XPoint *pathlist, short number, short style, float width)
 {
@@ -2148,50 +2273,89 @@ void strokepath(XPoint *pathlist, short number, short style, float width)
    char         dashstring[3];
    float        tmpwidth;
    short	minwidth;
+   static Pixmap pbuf = (Pixmap)NULL;
 
    tmpwidth = UTopTransScale(width);
    minwidth = max(1, (short)tmpwidth);
 
-   if (style & FILLED || (!(style & FILLED) && style & OPAQUE)) {
-      if ((style & FILLSOLID) == FILLSOLID)
-         SetFillStyle(dpy, areawin->gc, FillSolid);
-      else if (!(style & FILLED)) {
-         SetFillStyle(dpy, areawin->gc, FillOpaqueStippled); 
-	 SetStipple(dpy, areawin->gc, 7);
-      }
-      else {
-	 if (style & OPAQUE)
-            SetFillStyle(dpy, areawin->gc, FillOpaqueStippled);
-	 else
-            SetFillStyle(dpy, areawin->gc, FillStippled);
-         SetStipple(dpy, areawin->gc, ((style & FILLSOLID) >> 5));
-      }
-      FillPolygon(dpy, areawin->window, areawin->gc, pathlist, number, Nonconvex,
+   if (!(style & CLIPMASK) || (areawin->showclipmasks == TRUE) ||
+		(areawin->clipped < 0)) {
+      if (style & FILLED || (!(style & FILLED) && style & OPAQUE)) {
+         if ((style & FILLSOLID) == FILLSOLID)
+            SetFillStyle(dpy, areawin->gc, FillSolid);
+         else if (!(style & FILLED)) {
+            SetFillStyle(dpy, areawin->gc, FillOpaqueStippled); 
+	    SetStipple(dpy, areawin->gc, 7);
+         }
+         else {
+	    if (style & OPAQUE)
+               SetFillStyle(dpy, areawin->gc, FillOpaqueStippled);
+	    else
+               SetFillStyle(dpy, areawin->gc, FillStippled);
+            SetStipple(dpy, areawin->gc, ((style & FILLSOLID) >> 5));
+         }
+         FillPolygon(dpy, areawin->window, areawin->gc, pathlist, number, Nonconvex,
 		CoordModeOrigin);
-      /* return to original state */
-      SetFillStyle(dpy, areawin->gc, FillSolid);
-   }
-   if (!(style & NOBORDER)) {
-      /* set up dots or dashes */
-      if (style & DASHED) solidpart = (char)(4 * minwidth);
-      else if (style & DOTTED) solidpart = (char)minwidth;
-      sprintf(dashstring, "%c%c", solidpart, (char)(4 * minwidth));
-      if (style & (DASHED | DOTTED)) {
-         SetDashes(dpy, areawin->gc, 0, dashstring, 2);
-         SetLineAttributes(dpy, areawin->gc, tmpwidth, LineOnOffDash,
-		CapButt, (style & SQUARECAP) ? JoinMiter : JoinBevel);
+         /* return to original state */
+         SetFillStyle(dpy, areawin->gc, FillSolid);
       }
-      else
-         SetLineAttributes(dpy, areawin->gc, tmpwidth, LineSolid,
+      if (!(style & NOBORDER)) {
+         /* set up dots or dashes */
+         if (style & DASHED) solidpart = (char)(4 * minwidth);
+         else if (style & DOTTED) solidpart = (char)minwidth;
+         sprintf(dashstring, "%c%c", solidpart, (char)(4 * minwidth));
+         if (style & (DASHED | DOTTED)) {
+            SetDashes(dpy, areawin->gc, 0, dashstring, 2);
+            SetLineAttributes(dpy, areawin->gc, tmpwidth, LineOnOffDash,
+		CapButt, (style & SQUARECAP) ? JoinMiter : JoinBevel);
+         }
+         else
+            SetLineAttributes(dpy, areawin->gc, tmpwidth, LineSolid,
 		(style & SQUARECAP) ? CapProjecting : CapRound,
 		(style & SQUARECAP) ? JoinMiter : JoinBevel);
 
-      /* draw the spline and close off if so specified */
-      DrawLines(dpy, areawin->window, areawin->gc, pathlist,
+         /* draw the spline and close off if so specified */
+         DrawLines(dpy, areawin->window, areawin->gc, pathlist,
 		number, CoordModeOrigin);
-      if (!(style & UNCLOSED))
-         DrawLine(dpy, areawin->window, areawin->gc, pathlist[0].x,
+         if (!(style & UNCLOSED))
+            DrawLine(dpy, areawin->window, areawin->gc, pathlist[0].x,
 		pathlist[0].y, pathlist[number - 1].x, pathlist[number - 1].y);
+      }
+   }
+
+   if (style & CLIPMASK) {
+      if (areawin->clipped == 0) {
+         XSetForeground(dpy, areawin->cmgc, 0);
+         XFillRectangle(dpy, areawin->clipmask, areawin->cmgc, 0, 0,
+		areawin->width, areawin->height);
+         XSetForeground(dpy, areawin->cmgc, 1);
+         FillPolygon(dpy, areawin->clipmask, areawin->cmgc, pathlist,
+		number, Nonconvex, CoordModeOrigin);
+         XSetClipMask(dpy, areawin->gc, areawin->clipmask);
+	 // printf("level 0: Clip to clipmask\n");	// Diagnostic
+         areawin->clipped++;
+      }
+      else if ((areawin->clipped > 0) && (areawin->clipped & 1) == 0) {
+	 if (pbuf == (Pixmap)NULL) {
+	    pbuf = XCreatePixmap (dpy, areawin->window, areawin->width,
+			areawin->height, 1);
+	 }
+	 XCopyArea(dpy, areawin->clipmask, pbuf, areawin->cmgc,
+		0, 0, areawin->width, areawin->height, 0, 0);
+         XSetForeground(dpy, areawin->cmgc, 0);
+         XFillRectangle(dpy, areawin->clipmask, areawin->cmgc, 0, 0,
+		areawin->width, areawin->height);
+         XSetForeground(dpy, areawin->cmgc, 1);
+         FillPolygon(dpy, areawin->clipmask, areawin->cmgc, pathlist,
+		number, Nonconvex, CoordModeOrigin);
+         XSetFunction(dpy, areawin->cmgc, GXand);
+	 XCopyArea(dpy, pbuf, areawin->clipmask, areawin->cmgc,
+		0, 0, areawin->width, areawin->height, 0, 0);
+         XSetFunction(dpy, areawin->cmgc, GXcopy);
+         XSetClipMask(dpy, areawin->gc, areawin->clipmask);
+	 // printf("level X: Clip to clipmask\n");	// Diagnostic
+         areawin->clipped++;
+      }
    }
 }
 
@@ -2262,8 +2426,25 @@ void UDrawPath(pathptr thepath, float passwidth)
    polyptr	thepoly;
    splineptr	thespline;
    int		pathsegs = 0, curseg = 0;
-   float scaledwidth;
+   float	scaledwidth;
+   Boolean	draweditlines = FALSE;
    
+   /* First pass---check for any splines that are being edited.  If	*/
+   /* any one is, then draw all control points for all splines in the	*/
+   /* path.								*/
+
+   for (genpath = thepath->plist; genpath < thepath->plist + thepath->parts;
+	  genpath++) {
+      switch(ELEMENTTYPE(*genpath)) {
+	 case SPLINE:
+	    thespline = TOSPLINE(genpath);
+	    if (thespline->cycle != NULL)
+	       draweditlines = TRUE;
+	    break;
+      }
+      if (draweditlines) break;
+   }
+
    for (genpath = thepath->plist; genpath < thepath->plist + thepath->parts;
 	  genpath++) {
       switch(ELEMENTTYPE(*genpath)) {
@@ -2281,7 +2462,7 @@ void UDrawPath(pathptr thepath, float passwidth)
    	    makesplinepath(thespline, tmppoints + curseg);
 	    curseg = pathsegs;
 
-	    if (thespline->cycle != NULL) {
+	    if (draweditlines) {
 	       UDrawXLine(thespline->ctrl[0], thespline->ctrl[1]);  
 	       UDrawXLine(thespline->ctrl[3], thespline->ctrl[2]);
 	    }
@@ -2301,6 +2482,7 @@ void UDrawPath(pathptr thepath, float passwidth)
 /*    level is the level of recursion 					*/
 /*    passcolor is the inherited color value passed to object		*/
 /*    passwidth is the inherited linewidth value passed to the object	*/
+/*    stack contains graphics context information			*/
 /*----------------------------------------------------------------------*/
 
 void UDrawObject(objinstptr theinstance, short level, int passcolor,
@@ -2328,7 +2510,18 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor,
 
    UPushCTM();
 
-   if (stack) push_stack(stack, theinstance);
+   if (stack) {
+      /* Save the current clipping mask and push it on the stack */
+      if (areawin->clipped > 0) {
+         push_stack((pushlistptr *)stack, theinstance, (char *)areawin->clipmask);
+	 areawin->clipmask = XCreatePixmap(dpy, areawin->window, areawin->width,
+		areawin->height, 1);
+	 XCopyArea(dpy, (Pixmap)(*stack)->clientdata, areawin->clipmask, areawin->cmgc,
+		0, 0, areawin->width, areawin->height, 0, 0);
+      }
+      else
+         push_stack((pushlistptr *)stack, theinstance, (char *)NULL);
+   }
    if (level != 0)
        UPreMultCTM(DCTM, theinstance->position, theinstance->scale,
 			theinstance->rotation);
@@ -2369,13 +2562,21 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor,
        if ((*areagen)->type & DRAW_HIDE) continue;
 
        if (defaultcolor != DOFORALL) {
-
-	  if ((*areagen)->color != curcolor) {
-	     if ((*areagen)->color == DEFAULTCOLOR)
+	  Boolean clipcolor = FALSE;
+	  switch(ELEMENTTYPE(*areagen)) {
+ 	     case(POLYGON): case(SPLINE): case(ARC): case(PATH):
+	        if (TOPOLY(areagen)->style & CLIPMASK)
+		   clipcolor = TRUE;
+		break;
+	  }
+	  if (((*areagen)->color != curcolor) || (clipcolor == TRUE)) {
+	     if (clipcolor)
+		curcolor = CLIPMASKCOLOR;
+	     else if ((*areagen)->color == DEFAULTCOLOR)
 		curcolor = defaultcolor;
 	     else
 		curcolor = (*areagen)->color;
-	     /* XTopSetForeground(curcolor); */
+
 	     XcTopSetForeground(curcolor);
 	  }
        }
@@ -2415,7 +2616,9 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor,
 		}
 		if ((!alist) || (!blist)) break;
 	     }
+	     if (areawin->clipped > 0) areawin->clipped += 2;
              UDrawObject(TOOBJINST(areagen), level + 1, curcolor, passwidth, stack);
+	     if (areawin->clipped > 0) areawin->clipped -= 2;
 	     break;
    
   	  case(LABEL): 
@@ -2430,6 +2633,22 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor,
 		UDrawXDown(TOLABEL(areagen));
 	     break;
        }
+       if (areawin->clipped > 0) {
+	  if ((areawin->clipped & 3) == 1) {
+	     areawin->clipped++;
+	  }
+	  else if ((areawin->clipped & 3) == 2) {
+	     areawin->clipped -= 2;
+	     if ((!stack) || ((*stack)->clientdata == (char *)NULL)) {
+	        XSetClipMask(dpy, areawin->gc, None);
+	 	// printf("1: Clear clipmask\n");	// Diagnostic
+	     }
+	     else {
+	        XSetClipMask(dpy, areawin->gc, (Pixmap)((*stack)->clientdata));
+	 	// printf("1: Set to pushed clipmask\n");	// Diagnostic
+	     }
+	  }
+       }
      }
 
      /* restore the color passed to the object, if different from current color */
@@ -2437,12 +2656,34 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor,
      if ((defaultcolor != DOFORALL) && (passcolor != curcolor)) {
 	XTopSetForeground(passcolor);
      }
+     if (areawin->clipped > 0) {
+	if ((areawin->clipped & 3) != 3) {
+	   if ((!stack) || ((*stack)->clientdata == (char *)NULL)) {
+	      XSetClipMask(dpy, areawin->gc, None);
+	      // printf("2: Clear clipmask\n");	// Diagnostic
+	   }
+	   else {
+	      XSetClipMask(dpy, areawin->gc, (Pixmap)((*stack)->clientdata));
+	      // printf("2: Set to pushed clipmask\n");	// Diagnostic
+	   }
+	}
+	areawin->clipped &= ~3;
+     }
    }
 
-   /* restor the selection list (if any) */
+   /* restore the selection list (if any) */
    areawin->selects = savesel;
    UPopCTM();
-   if (stack) pop_stack(stack);
+   if (stack) {
+      if ((*stack) != NULL) {
+	 if ((*stack)->clientdata != (char *)NULL) {
+	    XFreePixmap(dpy, areawin->clipmask);
+            areawin->clipmask = (Pixmap)(*stack)->clientdata;
+	    // printf("3: Restore clipmask\n");	// Diagnostic
+	 }
+      }
+      pop_stack(stack);
+   }
 }
 
 /*----------------------------------------------------------------------*/
