@@ -1058,12 +1058,14 @@ XCWindowData *create_new_window()
    newwindow->lastlibrary = 0;
    newwindow->manhatn = False;
    newwindow->boxedit = MANHATTAN;
+   newwindow->pathedit = TANGENTS;
    newwindow->lastbackground = NULL;
    newwindow->editstack = (objectptr) malloc(sizeof(object));
    newwindow->stack = NULL;   /* at the top of the hierarchy */
    newwindow->hierstack = NULL;
    initmem(newwindow->editstack);
    newwindow->pinpointon = False;
+   newwindow->showclipmasks = True;
    newwindow->pinattach = False;
    newwindow->buschar = '(';	/* Vector notation for buses */
    newwindow->defaultcursor = &CROSS;
@@ -1073,6 +1075,9 @@ XCWindowData *create_new_window()
    newwindow->gccolor = 0;
    newwindow->gctype = 0;
    newwindow->time_id = 0;
+   newwindow->clipmask = (Pixmap)NULL;
+   newwindow->cmgc = (GC)NULL;
+   newwindow->clipped = 0;
    newwindow->vscale = 1;
    newwindow->pcorner.x = newwindow->pcorner.y = 0;
    newwindow->topinstance = (objinstptr)NULL;
@@ -1319,7 +1324,7 @@ nexthier:
 	       free_stack(&newstack);
 	       return TCL_ERROR;
 	   }
-	   push_stack(&newstack, chkinst);
+	   push_stack(&newstack, chkinst, NULL);
 	   goto nexthier;
         }
 	else
@@ -1449,6 +1454,17 @@ void post_initialize()
       dbuf = XCreatePixmap(dpy, areawin->window, areawin->width,
 		areawin->height, DefaultDepthOfScreen(xcScreen(areawin->area)));
 #endif
+   if (areawin->clipmask == (Pixmap)NULL)
+      areawin->clipmask = XCreatePixmap(dpy, areawin->window, areawin->width,
+		areawin->height, 1);
+
+   if (areawin->cmgc == (GC)NULL) {
+      XGCValues values;
+      values.foreground = 0;
+      values.background = 0;
+      areawin->cmgc = XCreateGC(dpy, areawin->clipmask,
+		GCForeground | GCBackground, &values);
+   }
 
 #ifdef TCL_WRAPPER
 
