@@ -455,6 +455,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define OPAQUE 256
 #define BBOX 512
 #define SQUARECAP 1024
+#define CLIPMASK  2048
 
 /*----------------------------------------------------------------------*/
 /* Box edit styles							*/
@@ -466,6 +467,12 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define RHOMBOIDY 4
 #define RHOMBOIDA 8
 
+/*----------------------------------------------------------------------*/
+/* Path edit styles							*/
+/*----------------------------------------------------------------------*/
+
+#define TANGENTS  1		/* (NORMAL = 0) */
+
 /*----------------------------------------------------------------------*/
 /* Cycle points (selected points) (flag bits, can be OR'd together)	*/
 /*----------------------------------------------------------------------*/
@@ -475,6 +482,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define LASTENTRY 0x04
 #define PROCESS	  0x08
 #define REFERENCE 0x10
+#define ANTIXY	  0x20		/* For matched-tangent curves */
 
 /*----------------------------------------------------------------------*/
 /* Arc creation and edit styles						*/
@@ -720,6 +728,7 @@ typedef struct _pushlist *pushlistptr;
 
 typedef struct _pushlist {
    objinstptr	thisinst;
+   char		*clientdata;	/* general-purpose record */
    pushlistptr	next;
 } pushlist;
 
@@ -779,9 +788,9 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   pointselect	*cycle;		/* Edit position(s), or NULL */
    u_short	style;
    float	width;
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    short	number;
    pointlist	points;
 } polygon, *polyptr;
@@ -794,9 +803,9 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   pointselect	*cycle;		/* Edit position(s), or NULL */
    u_short	style;
    float	width;
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    XPoint	ctrl[4];
    /* the following are for rendering only */
    XfPoint	points[INTSEGS];
@@ -810,9 +819,9 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   pointselect	*cycle;		/* Edit position(s), or NULL */
    u_short	style;
    float	width;
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    short	radius; 	/* x-axis radius */
    short	yaxis;		/* y-axis radius */
    float	angle1;		/* endpoint angles, in degrees */
@@ -1145,7 +1154,7 @@ typedef struct {
    Pixel	globalcolor, localcolor, infocolor, ratsnestcolor;
 
    /* non-schematic layout color(s) */
-   Pixel	bboxpix;
+   Pixel	bboxpix, clipcolor;
 
    /* color scheme 1 */
    Pixel	fg, bg;
@@ -1303,7 +1312,8 @@ enum {
    XCF_Cancel_Last /* 102 */,	XCF_Sim /* 103 */,
    XCF_SPICE /* 104 */,		XCF_PCB /* 105 */,
    XCF_SPICEflat /* 106 */,	XCF_Rescale /* 107 */,		
-   XCF_Text_Delete_Param /* 108 */,
+   XCF_Reorder /* 108 */,	XCF_Color /* 109 */,
+   XCF_Text_Delete_Param /* 110 */,
    NUM_FUNCTIONS
 };
 
@@ -1356,6 +1366,9 @@ typedef struct _windowdata {
    xcWidget	scrollbarh, scrollbarv;
    Window	window;
    GC 		gc;
+   Pixmap	clipmask;
+   char		clipped;
+   GC		cmgc;
    int		gccolor, gctype;
    xcIntervalId time_id;
 
@@ -1376,6 +1389,7 @@ typedef struct _windowdata {
    short	filter;		/* selection filter */
    Boolean	manhatn;
    Boolean	boxedit;
+   Boolean	pathedit;
    Boolean	snapto;
    Boolean	bboxon;
    Boolean	center;
@@ -1387,6 +1401,7 @@ typedef struct _windowdata {
    Boolean	editinplace;
    Boolean	pinpointon;
    Boolean	pinattach;	/* keep wires attached to pins when moving objinsts */
+   Boolean	showclipmasks;	/* draw clipmask shape outlines */
 #ifndef TCL_WRAPPER
 #ifdef HAVE_XPM
    Boolean	toolbar_on;
